@@ -1,7 +1,7 @@
 // samplv1.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -30,6 +30,8 @@
 
 #include "samplv1_fx.h"
 
+#include "samplv1_reverb.h"
+
 
 #ifdef CONFIG_DEBUG_0
 #include <stdio.h>
@@ -48,8 +50,8 @@
 const uint16_t MAX_VOICES = 32;			// polyphony
 const uint8_t  MAX_NOTES  = 128;
 
-const float MIN_ENV_MSECS = 2.0f;		// min 5msec per stage
-const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage
+const float MIN_ENV_MSECS = 2.0f;		// min 2msec per stage
+const float MAX_ENV_MSECS = 5000.0f;	// max 5 sec per stage (default)
 
 const float DETUNE_SCALE  = 0.5f;
 const float PHASE_SCALE   = 0.5f;
@@ -311,6 +313,7 @@ struct samplv1_gen
 	float *octave;
 	float *tuning;
 	float *glide;
+	float *envtime, envtime0;
 };
 
 
@@ -428,6 +431,18 @@ struct samplv1_del
 };
 
 
+// reverb
+
+struct samplv1_rev
+{
+	float *wet;
+	float *room;
+	float *damp;
+	float *feedb;
+	float *width;
+};
+
+
 // dynamic(compressor/limiter)
 
 struct samplv1_dyn
@@ -719,13 +734,15 @@ public:
 
 	void reset();
 
-	samplv1_sample gen1_sample;
-	samplv1_wave   lfo1_wave;
+	samplv1_sample  gen1_sample;
+	samplv1_wave_lf lfo1_wave;
 
 	float gen1_last;
 
 protected:
 
+	void updateEnvTimes();
+
 	void allSoundOff();
 	void allControllersOff();
 	void allNotesOff();
@@ -766,6 +783,7 @@ private:
 	samplv1_fla m_fla;
 	samplv1_pha m_pha;
 	samplv1_del m_del;
+	samplv1_rev m_rev;
 	samplv1_dyn m_dyn;
 
 	samplv1_voice **m_voices;
@@ -785,6 +803,8 @@ private:
 	samplv1_fx_phaser  *m_phaser;
 	samplv1_fx_delay   *m_delay;
 	samplv1_fx_comp    *m_comp;
+
+	samplv1_reverb m_reverb;
 };
 
 
@@ -811,6 +831,9 @@ samplv1_impl::samplv1_impl ( uint16_t iChannels, uint32_t iSampleRate )
 	// null sample.
 	m_gen1.sample0 = 0.0f;
 
+	// max env. stage length (default)
+	m_gen1.envtime0 = 0.0001f * MAX_ENV_MSECS;
+
 	// glide note.
 	gen1_last = 0.0f;
 
@@ -919,11 +942,29 @@ void samplv1_impl::setSampleRate ( uint32_t iSampleRate )
 	gen1_sample.setSampleRate(m_iSampleRate);
 	lfo1_wave.setSampleRate(m_iSampleRate);
 
+	updateEnvTimes();
+}
+
+
+uint32_t samplv1_impl::sampleRate (void) const
+{
+	return m_iSampleRate;
+}
+
+
+void samplv1_impl::updateEnvTimes (void)
+{
 	// update envelope range times in frames
 	const float srate_ms = 0.001f * float(m_iSampleRate);
 
+	float envtime_msecs = 10000.0f * m_gen1.envtime0;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = (gen1_sample.length() >> 1) / srate_ms;
+	if (envtime_msecs < MIN_ENV_MSECS)
+		envtime_msecs = MIN_ENV_MSECS + 1.0f;
+
 	const uint32_t min_frames = uint32_t(srate_ms * MIN_ENV_MSECS);
-	const uint32_t max_frames = uint32_t(srate_ms * MAX_ENV_MSECS);
+	const uint32_t max_frames = uint32_t(srate_ms * envtime_msecs);
 
 	m_dcf1.env.min_frames = min_frames;
 	m_dcf1.env.max_frames = max_frames;
@@ -936,12 +977,6 @@ void samplv1_impl::setSampleRate ( uint32_t iSampleRate )
 }
 
 
-uint32_t samplv1_impl::sampleRate (void) const
-{
-	return m_iSampleRate;
-}
-
-
 void samplv1_impl::setSampleFile ( const char *pszSampleFile )
 {
 	reset();
@@ -974,6 +1009,7 @@ void samplv1_impl::setParamPort ( samplv1::ParamIndex index, float *pfParam )
 	case samplv1::GEN1_OCTAVE:    m_gen1.octave      = pfParam; break;
 	case samplv1::GEN1_TUNING:    m_gen1.tuning      = pfParam; break;
 	case samplv1::GEN1_GLIDE:     m_gen1.glide       = pfParam; break;
+	case samplv1::GEN1_ENVTIME:   m_gen1.envtime     = pfParam; break;
 	case samplv1::DCF1_CUTOFF:    m_dcf1.cutoff      = pfParam; break;
 	case samplv1::DCF1_RESO:      m_dcf1.reso        = pfParam; break;
 	case samplv1::DCF1_TYPE:      m_dcf1.type        = pfParam; break;
@@ -1030,6 +1066,11 @@ void samplv1_impl::setParamPort ( samplv1::ParamIndex index, float *pfParam )
 	case samplv1::DEL1_BPM:       m_del.bpm          = pfParam; break;
 	case samplv1::DEL1_BPMSYNC:   m_del.bpmsync      = pfParam; break;
 	case samplv1::DEL1_BPMHOST:   m_del.bpmhost      = pfParam; break;
+	case samplv1::REV1_WET:       m_rev.wet          = pfParam; break;
+	case samplv1::REV1_ROOM:      m_rev.room         = pfParam; break;
+	case samplv1::REV1_DAMP:      m_rev.damp         = pfParam; break;
+	case samplv1::REV1_FEEDB:     m_rev.feedb        = pfParam; break;
+	case samplv1::REV1_WIDTH:     m_rev.width        = pfParam; break;
 	case samplv1::DYN1_COMPRESS:  m_dyn.compress     = pfParam; break;
 	case samplv1::DYN1_LIMITER:   m_dyn.limiter      = pfParam; break;
 	default: break;
@@ -1047,6 +1088,7 @@ float *samplv1_impl::paramPort ( samplv1::ParamIndex index )
 	case samplv1::GEN1_OCTAVE:    pfParam = m_gen1.octave;      break;
 	case samplv1::GEN1_TUNING:    pfParam = m_gen1.tuning;      break;
 	case samplv1::GEN1_GLIDE:     pfParam = m_gen1.glide;       break;
+	case samplv1::GEN1_ENVTIME:   pfParam = m_gen1.envtime;     break;
 	case samplv1::DCF1_CUTOFF:    pfParam = m_dcf1.cutoff;      break;
 	case samplv1::DCF1_RESO:      pfParam = m_dcf1.reso;        break;
 	case samplv1::DCF1_TYPE:      pfParam = m_dcf1.type;        break;
@@ -1103,6 +1145,11 @@ float *samplv1_impl::paramPort ( samplv1::ParamIndex index )
 	case samplv1::DEL1_BPM:       pfParam = m_del.bpm;          break;
 	case samplv1::DEL1_BPMSYNC:   pfParam = m_del.bpmsync;      break;
 	case samplv1::DEL1_BPMHOST:   pfParam = m_del.bpmhost;      break;
+	case samplv1::REV1_WET:       pfParam = m_rev.wet;          break;
+	case samplv1::REV1_ROOM:      pfParam = m_rev.room;         break;
+	case samplv1::REV1_DAMP:      pfParam = m_rev.damp;         break;
+	case samplv1::REV1_FEEDB:     pfParam = m_rev.feedb;        break;
+	case samplv1::REV1_WIDTH:     pfParam = m_rev.width;        break;
 	case samplv1::DYN1_COMPRESS:  pfParam = m_dyn.compress;     break;
 	case samplv1::DYN1_LIMITER:   pfParam = m_dyn.limiter;      break;
 	default: break;
@@ -1298,6 +1345,9 @@ void samplv1_impl::allSoundOff (void)
 		m_delay[k].reset();
 		m_comp[k].reset();
 	}
+
+	m_reverb.setSampleRate(m_iSampleRate);
+	m_reverb.reset();
 }
 
 
@@ -1368,6 +1418,9 @@ void samplv1_impl::reset (void)
 	if (m_comp == 0)
 		m_comp = new samplv1_fx_comp [m_iChannels];
 
+	// reverbs
+	m_reverb.reset();
+
 	allSoundOff();
 //	allControllersOff();
 	allNotesOff();
@@ -1395,19 +1448,28 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 	// controls
 
 	const float lfo1_rate = *m_lfo1.rate * *m_lfo1.rate;
-	const float lfo1_freq = LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
-	const float modwheel1 = m_ctl.modwheel + PITCH_SCALE * *m_lfo1.pitch;
+
+	const float lfo1_freq
+		= LFO_FREQ_MIN + lfo1_rate * (LFO_FREQ_MAX - LFO_FREQ_MIN);
+
+	const float modwheel1
+		= m_ctl.modwheel + PITCH_SCALE * *m_lfo1.pitch;
 
 	if (m_gen1.sample0 != *m_gen1.sample) {
 		m_gen1.sample0  = *m_gen1.sample;
 		gen1_sample.reset(samplv1_freq(m_gen1.sample0));
 	}
 
+	if (m_gen1.envtime0 != *m_gen1.envtime) {
+		m_gen1.envtime0  = *m_gen1.envtime;
+		updateEnvTimes();
+	}
+
 	if (bool(int(*m_gen1.loop)) != gen1_sample.isLoop())
 		gen1_sample.setLoop(bool(*m_gen1.loop));
 
-	if (int(*m_lfo1.shape) != int(lfo1_wave.shape()) || *m_lfo1.width != lfo1_wave.width())
-		lfo1_wave.reset(samplv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
+	lfo1_wave.reset_test(
+		samplv1_wave::Shape(*m_lfo1.shape), *m_lfo1.width);
 
 	// per voice
 
@@ -1538,16 +1600,16 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		m_del.bpm = del_bpmhost;
 	}
 
-	// effects
 
+	// chorus
+	if (m_iChannels > 1) {
+		m_chorus.process(outs[0], outs[1], nframes, *m_cho.wet,
+			*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
+	}
+
+	// effects
 	for (k = 0; k < m_iChannels; ++k) {
 		float *in = outs[k];
-		float *out = in;
-		// chorus
-		if (k > 0) {
-			m_chorus.process(outs[k - 1], outs[k], nframes, *m_cho.wet,
-				*m_cho.delay, *m_cho.feedb, *m_cho.rate, *m_cho.mod);
-		}
 		// flanger
 		m_flanger[k].process(in, nframes, *m_fla.wet,
 			*m_fla.delay, *m_fla.feedb, *m_fla.daft * float(k));
@@ -1557,11 +1619,23 @@ void samplv1_impl::process ( float **ins, float **outs, uint32_t nframes )
 		// delay
 		m_delay[k].process(in, nframes, *m_del.wet,
 			*m_del.delay, *m_del.feedb, *m_del.bpm);
+	}
+
+	// reverb
+	if (m_iChannels > 1) {
+		m_reverb.process(outs[0], outs[1], nframes, *m_rev.wet,
+			*m_rev.feedb, *m_rev.room, *m_rev.damp, *m_rev.width);
+	}
+
+	// dynamics
+	for (k = 0; k < m_iChannels; ++k) {
+		float *in = outs[k];
 		// compressor
 		if (int(*m_dyn.compress) > 0)
 			m_comp[k].process(in, nframes);
 		// limiter
 		if (int(*m_dyn.limiter) > 0) {
+			float *out = in;
 			for (uint32_t n = 0; n < nframes; ++n)
 				*out++ = samplv1_sigmoid(*in++);
 		}
