@@ -1,7 +1,7 @@
 // samplv1.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -68,6 +68,7 @@ public:
 		GEN1_OCTAVE,
 		GEN1_TUNING,
 		GEN1_GLIDE,
+		GEN1_ENVTIME,
 		DCF1_CUTOFF,
 		DCF1_RESO,
 		DCF1_TYPE,
@@ -126,6 +127,11 @@ public:
 		DEL1_BPM,
 		DEL1_BPMSYNC,
 		DEL1_BPMHOST,
+		REV1_WET,
+		REV1_ROOM,
+		REV1_DAMP,
+		REV1_FEEDB,
+		REV1_WIDTH,
 		DYN1_COMPRESS,
 		DYN1_LIMITER,
 
