@@ -1,7 +1,7 @@
 // samplv1_fx.h
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,12 +22,16 @@
 #ifndef __samplv1_fx_h
 #define __samplv1_fx_h
 
+#include <stdint.h>
+#include <stdlib.h>
+#include <math.h>
+
 
 //-------------------------------------------------------------------------
 // samplv1_fx
 //
 // -- borrowed, stirred and refactored from Highlife --
-// Copyright (C) 2007 arguru, discodsp.com
+//    Copyright (C) 2007 arguru, discodsp.com
 //
 
 //-------------------------------------------------------------------------
@@ -53,112 +57,110 @@ public:
 
 		// peaking, lowshelf and hishelf
 		if (type >= Peak) {
-			float amp   = ::powf(10.0f, (gain / 40.0f));
-			float omega = 2.0f * M_PI * freq / srate;
-			float tsin  = ::sinf(omega);
-			float tcos  = ::cosf(omega);
+			const float amp   = ::powf(10.0f, (gain / 40.0f));
+			const float omega = 2.0f * M_PI * freq / srate;
+			const float tsin  = ::sinf(omega);
+			const float tcos  = ::cosf(omega);
+			const float beta  = ::sqrtf(amp) / q;
 			if (bwq)
 				alpha = tsin * ::sinhf(::logf(2.0f) / 2.0f * q * omega / tsin);
 			else
 				alpha = tsin / (2.0f * q);
-
-			float beta = ::sqrtf(amp) / q;
-			// peaking
-			if (type == Peak) {
+			switch (type) {
+			case Peak:
+				// peaking
 				b0 =  1.0f + alpha * amp;
 				b1 = -2.0f * tcos;
 				b2 =  1.0f - alpha * amp;
 				a0 =  1.0f + alpha / amp;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha / amp;
-			}
-			else
-			// lowshelf
-			if (type == LoShelf) {
+				break;
+			case LoShelf:
+				// low-shelf
 				b0 = amp * ((amp + 1.0f) - (amp - 1.0f) * tcos + beta * tsin);
 				b1 = 2.0f * amp *((amp - 1.0f) - (amp + 1.0f) * tcos);
 				b2 = amp * ((amp + 1.0f) - (amp - 1.0f) * tcos - beta * tsin);
 				a0 = (amp + 1.0f) + (amp - 1.0f) * tcos + beta * tsin;
 				a1 = -2.0f *((amp - 1.0f) + (amp + 1.0f) * tcos);
 				a2 = (amp + 1.0f) + (amp - 1.0f) * tcos - beta * tsin;
-			}
-			else
-			// hishelf
-			if (type == HiShelf) {
+				break;
+			case HiShelf:
+			default:
+				// high-shelf
 				b0 = amp * ((amp + 1.0f) + (amp - 1.0f) * tcos + beta * tsin);
 				b1 = -2.0f * amp * ((amp - 1.0f) + (amp + 1.0f) * tcos);
 				b2 = amp * ((amp + 1.0f) + (amp - 1.0f) * tcos - beta * tsin);
 				a0 = (amp + 1.0f) - (amp - 1.0f) * tcos + beta * tsin;
 				a1 = 2.0f * ((amp - 1.0f) - (amp + 1.0f) * tcos);
 				a2 = (amp + 1.0f) - (amp - 1.0f) * tcos - beta * tsin;
+				break;
 			}
 		} else {
 			// other filters
-			float omega	= 2.0f * M_PI * freq / srate;
-			float tsin	= ::sinf(omega);
-			float tcos	= ::cosf(omega);
+			const float omega = 2.0f * M_PI * freq / srate;
+			const float tsin  = ::sinf(omega);
+			const float tcos  = ::cosf(omega);
 			if (bwq)
 				alpha = tsin * ::sinhf(::logf(2.0f) / 2.0f * q * omega / tsin);
 			else
 				alpha = tsin / (2.0f * q);
-			// lowpass
-			if (type == Low) {
+			switch (type) {
+			case Low:
+				// low-pass
 				b0 = (1.0f - tcos) / 2.0f;
 				b1 =  1.0f - tcos;
 				b2 = (1.0f - tcos) / 2.0f;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
-			}
-			else
-			// hipass
-			if (type == High) {
+				break;
+			case High:
+				// high-pass
 				b0 = (1.0f + tcos) / 2.0f;
 				b1 = -1.0f - tcos;
 				b2 = (1.0f + tcos) / 2.0f;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
-			}
-			else
-			// bandpass csg
-			if (type == Band1) {
+				break;
+			case Band1:
+				// band-pass csg
 				b0 =  tsin / 2.0f;
 				b1 =  0.0f;
 				b2 = -tsin / 2.0f;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
-			}
-			else
-			// bandpass czpg
-			if (type == Band2) {
+				break;
+			case Band2:
+				// band-pass czpg
 				b0 =  alpha;
 				b1 =  0.0f;
 				b2 = -alpha;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
-			}
-			else
-			// notch
-			if (type == Notch) {
+				break;
+			case Notch:
+				// notch
 				b0 =  1.0f;
 				b1 = -2.0f * tcos;
 				b2 =  1.0f;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
-			}
-			else
-			// allpass
-			if (type == AllPass) {
+				break;
+			case AllPass:
+			default:
+				// all-pass
 				b0 =  1.0f - alpha;
 				b1 = -2.0f * tcos;
 				b2 =  1.0f + alpha;
 				a0 =  1.0f + alpha;
 				a1 = -2.0f * tcos;
 				a2 =  1.0f - alpha;
+				break;
 			}
 		}
 		// set filter coeffs
@@ -226,7 +228,7 @@ public:
 		m_attack  = ::expf(-1000.0f / (m_srate * 3.6f));
 		m_release = ::expf(-1000.0f / (m_srate * 150.0f));
 
-		// HOUSE eq.
+		// rock-da-house eq.
 		m_lo.reset(samplv1_fx_filter::Peak,      100.0f, m_srate, 1.0f, 6.0f);
 		m_mi.reset(samplv1_fx_filter::LoShelf,  1000.0f, m_srate, 1.0f, 3.0f);
 		m_hi.reset(samplv1_fx_filter::HiShelf, 10000.0f, m_srate, 1.0f, 4.0f);
@@ -281,9 +283,6 @@ class samplv1_fx_flanger
 {
 public:
 
-	static const uint32_t MAX_SIZE = 4096;	//= (1 << 12);
-	static const uint32_t MAX_MASK = MAX_SIZE - 1;
-
 	samplv1_fx_flanger()
 		{ reset(); }
 
@@ -340,6 +339,9 @@ public:
 			in[i] += wet * output(in[i], delay, feedb);
 	}
 
+	static const uint32_t MAX_SIZE = (1 << 12);	//= 4096;
+	static const uint32_t MAX_MASK = MAX_SIZE - 1;
+
 private:
 
 	float m_buffer[MAX_SIZE];
@@ -399,12 +401,13 @@ public:
 		}
 	}
 
+protected:
+
 	float pseudo_sinf(float x) const
 	{
 		x *= x;
 		x -= 1.0f;
-		x *= x;
-		return x;
+		return x * x;
 	}
 
 private:
@@ -425,9 +428,6 @@ class samplv1_fx_delay
 {
 public:
 
-	static const uint32_t MAX_SIZE = 65536;	//= (1 << 16);
-	static const uint32_t MAX_MASK = MAX_SIZE - 1;
-
 	samplv1_fx_delay(uint32_t iSampleRate = 44100)
 		: m_srate(iSampleRate) { reset(); }
 
@@ -459,8 +459,9 @@ public:
 		// set integer delay
 		uint32_t ndelay = uint32_t(delay_time);
 		// clamp
-		if (ndelay < 256)
-			ndelay = 256;
+		if (ndelay < MIN_SIZE)
+			ndelay = MIN_SIZE;
+		else
 		if (ndelay > MAX_SIZE)
 			ndelay = MAX_SIZE;
 		// delay process
@@ -472,6 +473,10 @@ public:
 		}
 	}
 
+	static const uint32_t MIN_SIZE = (1 <<  8);	//= 256;
+	static const uint32_t MAX_SIZE = (1 << 16);	//= 65536;
+	static const uint32_t MAX_MASK = MAX_SIZE - 1;
+
 private:
 
 	float m_srate;
@@ -517,8 +522,6 @@ class samplv1_fx_phaser
 {
 public:
 
-	static const uint16_t MAX_TAPS = 6;
-
 	samplv1_fx_phaser(uint32_t iSampleRate = 44100)
 		: m_srate(float(iSampleRate)) { reset(); }
 
@@ -563,10 +566,8 @@ public:
 			// positive wrap phase
 			if (m_lfo_phase >= 2.0f * M_PI)
 				m_lfo_phase -= 2.0f * M_PI;
-			// anti-denormalizer noise
-			const float ad = 1E-14f * float(::rand());
 			// get input
-			m_out = in[i] + ad + m_out * feedb;
+			m_out = denormal(in[i] + m_out * feedb);
 			// update filter coeffs and calculate output
 			for (uint16_t n = 0; n < MAX_TAPS; ++n)
 				m_out = m_taps[n].output(m_out, delay);
@@ -575,6 +576,17 @@ public:
 		}
 	}
 
+	static const uint16_t MAX_TAPS = 6;
+
+protected:
+
+	static float denormal(float v)
+	{
+		union { float f; unsigned int w; } u;
+		u.f = v;
+		return (u.w & 0x7f800000) ? v : 0.0f;
+	}
+
 private:
 
 	float m_srate;
