@@ -1,7 +1,7 @@
 // samplv1_param.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ struct {
 	{ "GEN1_OCTAVE",    0.0f },
 	{ "GEN1_TUNING",    0.0f },
 	{ "GEN1_GLIDE",     0.0f },
+	{ "GEN1_ENVTIME",   0.5f },
 	{ "DCF1_CUTOFF",    1.0f }, // 0.5f
 	{ "DCF1_RESO",      0.0f },
 	{ "DCF1_TYPE",      0.0f },
@@ -96,12 +97,17 @@ struct {
 	{ "DEL1_BPM",     180.0f },
 	{ "DEL1_BPMSYNC",   0.0f },
 	{ "DEL1_BPMHOST", 180.0f },
+	{ "REV1_WET",       0.0f },
+	{ "REV1_ROOM",      0.5f },
+	{ "REV1_DAMP",      0.5f },
+	{ "REV1_FEEDB",     0.5f },
+	{ "REV1_WIDTH",     0.0f },
 	{ "DYN1_COMPRESS",  0.0f },
-	{ "DYN1_LIMIT",     1.0f }
+	{ "DYN1_LIMITER",   1.0f }
 };
 
 
-QString samplv1_param::paramName ( samplv1::ParamIndex index )
+const char *samplv1_param::paramName ( samplv1::ParamIndex index )
 {
 	return samplv1_default_params[index].name;
 }
