@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -143,8 +143,11 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Fla1WetKnob->setSpecialValueText(sOff);
 	m_ui.Pha1WetKnob->setSpecialValueText(sOff);
 	m_ui.Del1WetKnob->setSpecialValueText(sOff);
+	m_ui.Rev1WetKnob->setSpecialValueText(sOff);
 
-	m_ui.Del1BpmKnob->setSpecialValueText(tr("Auto"));
+	const QString& sAuto = tr("Auto");
+	m_ui.Gen1EnvTimeKnob->setSpecialValueText(sAuto);
+	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
 	// GEN note limits.
 	m_ui.Gen1SampleKnob->setMinimum(0.0f);
@@ -200,13 +203,17 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Del1BpmKnob->setMinimum(3.6f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
 
+	// Reverb (stereo-)width limits.
+	m_ui.Rev1WidthKnob->setMinimum(-1.0f);
+	m_ui.Rev1WidthKnob->setMaximum(+1.0f);
 
 	// GEN1
-	setParamKnob(samplv1::GEN1_SAMPLE, m_ui.Gen1SampleKnob);
-	setParamKnob(samplv1::GEN1_LOOP,   m_ui.Gen1LoopKnob);
-	setParamKnob(samplv1::GEN1_OCTAVE, m_ui.Gen1OctaveKnob);
-	setParamKnob(samplv1::GEN1_TUNING, m_ui.Gen1TuningKnob);
-	setParamKnob(samplv1::GEN1_GLIDE,  m_ui.Gen1GlideKnob);
+	setParamKnob(samplv1::GEN1_SAMPLE,  m_ui.Gen1SampleKnob);
+	setParamKnob(samplv1::GEN1_LOOP,    m_ui.Gen1LoopKnob);
+	setParamKnob(samplv1::GEN1_OCTAVE,  m_ui.Gen1OctaveKnob);
+	setParamKnob(samplv1::GEN1_TUNING,  m_ui.Gen1TuningKnob);
+	setParamKnob(samplv1::GEN1_GLIDE,   m_ui.Gen1GlideKnob);
+	setParamKnob(samplv1::GEN1_ENVTIME, m_ui.Gen1EnvTimeKnob);
 
 	// DCF1
 	setParamKnob(samplv1::DCF1_CUTOFF,   m_ui.Dcf1CutoffKnob);
@@ -400,6 +407,13 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 		SIGNAL(valueChanged(float)),
 		SLOT(bpmSyncChanged()));
 
+	// Reverb
+	setParamKnob(samplv1::REV1_WET,   m_ui.Rev1WetKnob);
+	setParamKnob(samplv1::REV1_ROOM,  m_ui.Rev1RoomKnob);
+	setParamKnob(samplv1::REV1_DAMP,  m_ui.Rev1DampKnob);
+	setParamKnob(samplv1::REV1_FEEDB, m_ui.Rev1FeedbKnob);
+	setParamKnob(samplv1::REV1_WIDTH, m_ui.Rev1WidthKnob);
+
 	// Dynamics
 	setParamKnob(samplv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(samplv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
@@ -474,6 +488,8 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 // Param kbob (widget) map accesors.
 void samplv1widget::setParamKnob ( samplv1::ParamIndex index, samplv1widget_knob *pKnob )
 {
+	pKnob->setDefaultValue(samplv1_param::paramDefaultValue(index));
+
 	m_paramKnobs.insert(index, pKnob);
 	m_knobParams.insert(pKnob, index);
 
@@ -489,13 +505,14 @@ samplv1widget_knob *samplv1widget::paramKnob ( samplv1::ParamIndex index ) const
 
 
 // Param port accessors.
-void samplv1widget::setParamValue ( samplv1::ParamIndex index, float fValue )
+void samplv1widget::setParamValue (
+	samplv1::ParamIndex index, float fValue, bool bDefault )
 {
 	++m_iUpdate;
 
 	samplv1widget_knob *pKnob = paramKnob(index);
 	if (pKnob)
-		pKnob->setValue(fValue);
+		pKnob->setValue(fValue, bDefault);
 
 	updateParamEx(index, fValue);
 
@@ -646,8 +663,8 @@ void samplv1widget::resetParamValues (void)
 
 	for (uint32_t i = 0; i < samplv1::NUM_PARAMS; ++i) {
 		samplv1::ParamIndex index = samplv1::ParamIndex(i);
-		float fValue = samplv1_param::paramDefaultValue(index);
-		setParamValue(index, fValue);
+		const float fValue = samplv1_param::paramDefaultValue(index);
+		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 		m_params_ab[index] = fValue;
 	}
@@ -719,8 +736,8 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 
 	clearSampleFile();
 
-	resetParamValues();
 	resetParamKnobs();
+	resetParamValues();
 
 	const QFileInfo fi(sFilename);
 	const QDir currentDir(QDir::current());
@@ -752,14 +769,17 @@ void samplv1widget::loadPreset ( const QString& sFilename )
 							samplv1::ParamIndex index = samplv1::ParamIndex(
 								eParam.attribute("index").toULong());
 							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty() && s_hash.contains(sName))
+							if (!sName.isEmpty()) {
+								if (!s_hash.contains(sName))
+									continue;
 								index = s_hash.value(sName);
+							}
 							float fValue = eParam.text().toFloat();
 						//--legacy support < 0.3.0.4 -- begin
 							if (index == samplv1::DEL1_BPM && fValue < 3.6f)
 								fValue *= 100.0f;
 						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue);
+							setParamValue(index, fValue, true);
 							updateParam(index, fValue);
 							m_params_ab[index] = fValue;
 						}
