@@ -1,7 +1,7 @@
 // samplv1widget_jack.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -84,6 +84,9 @@ samplv1widget_jack::samplv1widget_jack ( samplv1_jack *pSampl )
 			SIGNAL(hide()),
 			SLOT(hideSession()));
 		m_pNsmClient->announce(SAMPLV1_TITLE, ":switch:dirty:optional-gui:");
+		samplv1widget_config *pConfig = samplv1widget_config::getInstance();
+		if (pConfig)
+			pConfig->bDontUseNativeDialogs = true;
 		return;
 	}
 #endif	// CONFIG_NSM
@@ -113,7 +116,7 @@ samplv1widget_jack::samplv1widget_jack ( samplv1_jack *pSampl )
 samplv1widget_jack::~samplv1widget_jack (void)
 {
 	m_pSampl->deactivate();
-	m_pSampl->close();
+//	m_pSampl->close();
 }
 
 
@@ -189,9 +192,12 @@ void samplv1widget_jack::openSession (void)
 	m_pSampl->deactivate();
 	m_pSampl->close();
 
+	const QString& client_id = m_pNsmClient->client_id();
 	const QString& path_name = m_pNsmClient->path_name();
 	const QString& display_name = m_pNsmClient->display_name();
-	const QString& client_id = m_pNsmClient->client_id();
+
+	m_pSampl->open(client_id.toUtf8().constData());
+	m_pSampl->activate();
 
 	const QDir dir(path_name);
 	if (!dir.exists())
@@ -201,9 +207,6 @@ void samplv1widget_jack::openSession (void)
 	if (fi.exists())
 		loadPreset(fi.absoluteFilePath());
 
-	m_pSampl->open(client_id.toUtf8().constData());
-	m_pSampl->activate();
-
 	m_bNsmDirty = false;
 
 	m_pNsmClient->open_reply();
