@@ -1,7 +1,7 @@
 // samplv1widget_sample.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -46,7 +46,7 @@ samplv1widget_sample::samplv1widget_sample (
 {
 	QFrame::setMouseTracking(true);
 	QFrame::setFocusPolicy(Qt::ClickFocus);
-	QFrame::setMinimumSize(QSize(580, 80));
+	QFrame::setMinimumSize(QSize(520, 80));
 
 	QFrame::setFrameShape(QFrame::Panel);
 	QFrame::setFrameShadow(QFrame::Sunken);
@@ -524,8 +524,11 @@ void samplv1widget_sample::openSample (void)
 	const QString& sTitle  = tr("Open Sample") + " - " SAMPLV1_TITLE;
 	const QString& sFilter = s_filters.join(";;");
 #if 1//QT_VERSION < 0x040400
+	QFileDialog::Options options = 0;
+	if (pConfig->bDontUseNativeDialogs)
+		options |= QFileDialog::DontUseNativeDialog;
 	sFilename = QFileDialog::getOpenFileName(parentWidget(),
-		sTitle, pConfig->sSampleDir, sFilter);
+		sTitle, pConfig->sSampleDir, sFilter, NULL, options);
 #else
 	QFileDialog fileDialog(nativeParentWidget(),
 		sTitle, pConfig->sSampleDir, sFilter);
@@ -534,6 +537,8 @@ void samplv1widget_sample::openSample (void)
 	QList<QUrl> urls(fileDialog.sidebarUrls());
 	urls.append(QUrl::fromLocalFile(pConfig->sSampleDir));
 	fileDialog.setSidebarUrls(urls);
+	if (pConfig->bDontUseNativeDialogs)
+		fileDialog.setOptions(QFileDialog::DontUseNativeDialog);
 	if (fileDialog.exec())
 		sFilename = fileDialog.selectedFiles().first();
 #endif
