@@ -1,7 +1,7 @@
 // samplv1widget_wave.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2014, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 #include <QMouseEvent>
 #include <QWheelEvent>
 
+#include <math.h>
 
 
 // Safe value capping.
@@ -45,7 +46,7 @@ samplv1widget_wave::samplv1widget_wave (
 	: QFrame(pParent, wflags),
 		m_bDragging(false), m_iDragShape(0)
 {
-	m_pWave = new samplv1_wave(128, 0);
+	m_pWave = new samplv1_wave_lf();
 
 	setFixedSize(QSize(60, 60));
 
